package edu.kit.ipd.descartes.adaptation.util;

import edu.kit.ipd.descartes.adaptation.TacticExecutor;
import edu.kit.ipd.descartes.mm.adaptation.WeightedTactic;
import edu.kit.ipd.descartes.mm.resourcelandscape.ComputingInfrastructure;
import edu.kit.ipd.descartes.mm.resourcelandscape.DistributedDataCenter;

public class BlueYonderExperimentTracker implements IObserver {

    private static final String RESULT_NAME_TEMPLATE = "Xreq-par-$1Yps-desc4-gw-desc3-db-desc2-predict-Z ps=$2-req=5-size=50000";
//    private static final String RESULT_NAME_TEMPLATE = "Xreq-par-$1$3Yps-desc4-gw-desc3-db-desc2-predict-Z ps=$2-req=40-size=500000";
    private static BlueYonderExperimentTracker instance = null;
    private int psOnDescOne = 0;
    private int psOnDescFour = 1;
    private int psOnDescOneBackup;
    private int psOnDescFourBackup;

    public void setSubject(TacticExecutor te) {
        te.register(this);
    }

    public String getCurrentExperimentName() {
        String result = RESULT_NAME_TEMPLATE;

        if (psOnDescOne > 0)
            result = RESULT_NAME_TEMPLATE.replace("$1", psOnDescOne + "ps-desc1-");
        else
            result = RESULT_NAME_TEMPLATE.replace("$1", "");

        result = result.replace("$2", Integer.toString(psOnDescFour));
        return result;
    }

    public void track(WeightedTactic tactic) {
        String execTacticName = tactic.getName();
        backupValues();

        if (execTacticName.contains("Consolidate")) {
            int loopCount = 1;
            try {
                loopCount = Integer.parseInt(tactic.getUsedTactic().getInputParams().get(0).getValue());
            } catch (Exception e) {
                
            }
            if (execTacticName.contains("Low")) {
                psOnDescFour = psOnDescFour - loopCount;
                psOnDescOne = psOnDescOne + loopCount;
            } else if (execTacticName.contains("High")) {
                psOnDescOne = psOnDescOne - loopCount;
                psOnDescFour = psOnDescFour + loopCount;
            }
        } else if (execTacticName.contains("Low")) {
            psOnDescOne++;
            if (execTacticName.contains("Migrate")) {
                assert psOnDescFour > 0;
                psOnDescFour--;
            }
        } else if (execTacticName.contains("High")) {
            psOnDescFour++;
            if (execTacticName.contains("Migrate")) {
                assert psOnDescOne > 0;
                psOnDescOne--;
            }
        }
    }

    private void backupValues() {
        psOnDescOneBackup = psOnDescOne;
        psOnDescFourBackup = psOnDescFour;
    }

    @Override
    public void undo() {
        resetValues();
    }

    private void resetValues() {
        psOnDescOne = psOnDescOneBackup;
        psOnDescFour = psOnDescFourBackup;
    }

    public static BlueYonderExperimentTracker getInstance() {
        if (instance == null)
            instance = new BlueYonderExperimentTracker();
        return instance;
    }

    public String getExperimentNameForDistributedDataCenter(DistributedDataCenter distributedDataCenter) {
        int[] psInstances = new int[4];
        
        for (int i = 0; i < psInstances.length; i++)
        {
            try {
                psInstances[i] = ((ComputingInfrastructure) distributedDataCenter.getConsistsOf().get(0).getContains().get(i)).getContains().size();
            } catch (Exception e) {
                psInstances[i] = 0;
            }
        }
        
        String result = RESULT_NAME_TEMPLATE;

        if (psInstances[0] > 0)
            result = result.replace("$1", psInstances[0] + "ps-desc1-");
        else
            result = result.replace("$1", "");
        
        if (psInstances[1] > 0)
            result = result.replace("$3", psInstances[1] + "ps-desc2-");
        else
            result = result.replace("$3", "");

        result = result.replace("$2", Integer.toString(psInstances[3]));
        return result;
    }
}
